Usage
track.rebuild(pos = 1, envir = as.environment(pos), dir = NULL,
fix = FALSE, level=c("missing", "all"), trust=c("unspecified", "environment", "db"),
verbose=1, RDataSuffix=NULL, dryRun=TRUE, replace.wd=TRUE,
use.file.times=TRUE)Arguments
pos
The position of the environment to which the tracking database to rebuild is linked (for active tracking databases.
envir
The tracked environment for which to rebuild the tracking database (for active tracking databases.
dir
The directory where the tracking database is stored (for tracking databases not currently active.
fix
If TRUE, try to fix various problems such as
illegal file names by renaming files or moving unusable
RData files to a 'quarantine' directory. If fix=TRUE and
dryRun=TRUE, changes will be reported but
level
If "missing", rebuild only missing information, if
"all", rebuild information for all objects.
trust
When rebuilding an active tracking database, and there
are conflicts between the data in the environment versus that in
files, which should be trusted?
verbose
Controls number of messages about progress
RDataSuffix
What suffix should be used for RData files?
(Usually worked out automatically.)
dryRun
If TRUE, no data objects in R or files on disk
are changed: the return value is a list containing the rebuilt file map and
the rebuilt object summary. If FALSE, objects in R
environments and files on disk can be c
replace.wd
If TRUE, file paths in diagnostic output are
printed starting with "./" where possible -- this makes comparisons
of test output more portable.
use.file.times
If TRUE, file creation, modification and
access times are used for objects that were not found in
existing summary objects. If FALSE, the current time is
used, which can be useful for testing purposes.